//----------------------------------------------------------------------------
// File: WinMain.cpp
//
// Desc: This sample shows how to enumerate DirectSound sound and capture
//       devices.
//
// Copyright (c) 1999 Microsoft Corp. All rights reserved.
// Ported by Vadim Bodrov, TMT Development Corp.
//-----------------------------------------------------------------------------
uses Windows, Strings, Messages, DXLite;

{$r enumdev.res}

const
    IDR_MAINFRAME                  = 128;
    IDD_MAIN                       = 130;
    IDC_CREATE                     = 1000;
    IDC_SOUND_DEVICE_COMBO         = 1010;
    IDC_CAPTURE_DEVICE_COMBO       = 1011;

// Next default values for new objects
   _APS_NEXT_RESOURCE_VALUE        = 131;
   _APS_NEXT_COMMAND_VALUE         = 32771;
   _APS_NEXT_CONTROL_VALUE         = 1019;
   _APS_NEXT_SYMED_VALUE           = 101;

//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------

var
    g_pDS: PPDirectSound := nil;
    g_pDSCapture: PPDirectSoundCapture := nil;



//-----------------------------------------------------------------------------
// Name: FreeDirectSound()
// Desc: Releases DirectSound
//-----------------------------------------------------------------------------
function FreeDirectSound: HResult;
begin
    // Release DirectSound interfaces
    if g_pDSCapture <> nil then
    begin
        g_pDSCapture^^.Release^(g_pDSCapture);
        g_pDSCapture := nil
    end;
    if g_pDS <> nil then
    begin
        g_pDS^^.Release^(g_pDS);
        g_pDS := nil
    end;

    Result := S_OK;
end;



//-----------------------------------------------------------------------------
// Name: InitDirectSound()
// Desc: Initilizes DirectSound
//-----------------------------------------------------------------------------
function InitDirectSound( hDlg: HWND ): HResult;
var
// Get the HWNDs the combo boxes
    hSoundDeviceCombo: HWND;
    hCaptureDeviceCombo: HWND;
// Get the index of the currently selected devices
    nSoundIndex: Longint;
    nCaptureIndex: Longint;
    pSoundGUID:   ^TGUID;
    pCaptureGUID: ^TGUID;
begin
    // Free any previous DirectSound objects
    FreeDirectSound;

    // Get the HWNDs the combo boxes
    hSoundDeviceCombo := GetDlgItem( hDlg, IDC_SOUND_DEVICE_COMBO );
    hCaptureDeviceCombo:= GetDlgItem( hDlg, IDC_CAPTURE_DEVICE_COMBO );

    // Get the index of the currently selected devices
    nSoundIndex := SendMessage( hSoundDeviceCombo,   CB_GETCURSEL, 0, 0 );
    nCaptureIndex := SendMessage( hCaptureDeviceCombo, CB_GETCURSEL, 0, 0 );

    // Get the GUID attached to the combo box item
    pSoundGUID := PGUID(SendMessage( hSoundDeviceCombo, CB_GETITEMDATA,
        nSoundIndex, 0 ));
    pCaptureGUID := PGUID(SendMessage( hCaptureDeviceCombo, CB_GETITEMDATA,
        nCaptureIndex, 0 ));

    // Create IDirectSound using the select sound device

    Result := DirectSoundCreate(pSoundGUID, g_pDS, nil );
    if FAILED( Result ) then
        exit;

    // Release the IDirectSound object immediately since we don't want
    // to limit this sample to only computers that support full duplex audio
    if g_pDS <> nil then
    begin
        g_pDS^^.Release^(g_pDS);
        g_pDS := nil
    end;

    // Create IDirectSoundCapture using the select capture device
    Result := DirectSoundCaptureCreate( pCaptureGUID, g_pDSCapture, nil );
    if FAILED( Result ) then
        exit;

    // Release g_pDSCapture, since we don't need it really
    if g_pDSCapture <> nil then
    begin
        g_pDSCapture^^.Release^(g_pDSCapture);
        g_pDSCapture := nil
    end;

    Result := S_OK;
end;


var
    // Set aside static storage space for 20 audio drivers
    AudioDriverGUIDs: array[0..19] of TGUID;
    dwAudioDriverIndex: DWORD := 0;

//-----------------------------------------------------------------------------
// Name: DSoundEnumCallback()
// Desc: Enumeration callback called by DirectSoundEnumerate
//-----------------------------------------------------------------------------
function DSoundEnumCallback conv arg_stdcall ( pGUID: PGUID; strDesc, strDrvName: LPSTR;
    pContext: Pointer ): LONGBOOL;
var
    pTemp: ^TGUID := nil;
    hSoundDeviceCombo: HWND;
    nIndex: Longint;
begin
    if pGUID <> nil then
    begin
        if dwAudioDriverIndex >= 20 then
        begin
            Result := LONGTRUE;
            exit;
        end;

        pTemp := @AudioDriverGUIDs[dwAudioDriverIndex];
        dwAudioDriverIndex +:= 1;
        CopyMemory( pTemp, pGUID, sizeof(TGUID) );
    end;

    hSoundDeviceCombo := HWND(pContext);

    // Add the string to the combo box
    SendMessage( hSoundDeviceCombo, CB_ADDSTRING,
                 0, DWORD(strDesc) );

    // Get the index of the string in the combo box
    nIndex := SendMessage( hSoundDeviceCombo, CB_FINDSTRING,
                 0, DWORD(strDesc) );

    // Set the item data to a pointer to the static guid stored in AudioDriverGUIDs
    SendMessage( hSoundDeviceCombo, CB_SETITEMDATA,
                 nIndex, DWORD(pTemp) );

    Result := LONGTRUE;
end;




//-----------------------------------------------------------------------------
// Name: OnInitDialog()
// Desc: Initializes the dialogs (sets up UI controls, etc.)
//-----------------------------------------------------------------------------
function OnInitDialog( hDlg: HWND ): HResult;
var
    // Load the icon
    hInst: THandle := GetWindowLong( hDlg, GWL_HINSTANCE );
    Icon: HIcon := LoadIcon( hInst, MAKEINTRESOURCE( IDR_MAINFRAME ) );
    hSoundDeviceCombo: HWND;
    hCaptureDeviceCombo: HWND;
begin

    // Set the icon for this dialog.
    PostMessage( hDlg, WM_SETICON, ICON_BIG,   Icon );  // Set big icon
    PostMessage( hDlg, WM_SETICON, ICON_SMALL, Icon );  // Set small icon

    // Enumerate the sound devices and place them in the combo box
    hSoundDeviceCombo := GetDlgItem( hDlg, IDC_SOUND_DEVICE_COMBO );
    Result := DirectSoundEnumerate( TDSEnumCallback(@DSoundEnumCallback), Pointer(hSoundDeviceCombo) );
    if FAILED( Result ) then
        exit;

    // Enumerate the capture devices and place them in the combo box
    hCaptureDeviceCombo := GetDlgItem( hDlg, IDC_CAPTURE_DEVICE_COMBO );
    Result := DirectSoundCaptureEnumerate(TDSEnumCallback(@DSoundEnumCallback), Pointer(hCaptureDeviceCombo) );
    if FAILED( Result ) then
        exit;

    // Select the first device in the combo box
    SendMessage( hSoundDeviceCombo,   CB_SETCURSEL, 0, 0 );
    SendMessage( hCaptureDeviceCombo, CB_SETCURSEL, 0, 0 );

    Result := S_OK;
end;





//-----------------------------------------------------------------------------
// Name: MainDlgProc()
// Desc: Handles dialog messages
//-----------------------------------------------------------------------------
function MainDlgProc conv arg_stdcall (hDlg: HWND; msg: UINT; wParam: WPARAM; lParam:LPARAM): LResult;
begin
    case msg of
        WM_COMMAND:
        begin
            case LOWORD(wParam) of
                IDC_CREATE:
                begin
                    // Init DirectSound
                    if SUCCEEDED( InitDirectSound( hDlg ) ) then
                        MessageBox( hDlg, 'DirectSound interface created successfully',
                                          'EnumDevices', MB_OK )
                    else
                        MessageBox( hDlg, 'DirectSound interface creatation failed',
                                          'EnumDevices', MB_OK or MB_ICONERROR );
                end;

                IDCANCEL:
                    EndDialog( hDlg, IDCANCEL );

            else
                begin
                    Result := LONGFALSE; // Didn't handle message
                    exit;
                end;
            end;
        end;

        WM_INITDIALOG:
            if FAILED( OnInitDialog( hDlg ) ) then
            begin
                MessageBox( hDlg, 'Error enumerating DirectSound devices.' + #13 +
                                  'Sample will now exit.', 'DirectSound Sample',
                                  MB_OK or MB_ICONERROR );
                EndDialog( hDlg, IDABORT );
            end;

        WM_DESTROY:
            // Cleanup everything
            FreeDirectSound;

        else
            begin
               Result := LONGFALSE; // Didn't handle message
               exit;
            end;
    end;

    Result := LONGTRUE; // Handled message
end;



begin
    // Initialize COM
    CoInitialize( nil );

    // Display the main dialog box.
    DialogBox( hInstance, MAKEINTRESOURCE(IDD_MAIN), 0, @MainDlgProc );

    // Release COM
    CoUninitialize;
end.